﻿/***************************************************************
 *        WebSocket 基本应用示例
 * =============================================================   
 * 本DEMO的目的意义：
 *  演示如果直接利用OWIN字典进行WebSocket应用开发
 *  
 *  使用方法：将编译得到的dll放到网站的bin文件夹中。
 * *************************************************************/


#region <USING>

using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using System.Linq;
using System.Threading;

#endregion


namespace WebSocket.Demo
{


    public class Adapter
    {


        /// <summary>
        /// OWIN适配器的主函数
        /// </summary>
        /// <param name="env"></param>
        /// <returns></returns>
        public Task OwinMain(IDictionary<string, object> env)
        {
            //是否包含Websocket握手函数并尝试进行WebSocket连接
            if (env.ContainsKey("websocket.Accept"))
            {
                var x = new ClientDemo(env);
                if (x.Accept()) return Task.Delay(0);
            }

            //如果不是websocket请求，就接普通OWIN处理
            return ProcessRequest(env);
        }


        /// <summary>
        /// 你自己对请求的处理函数
        /// </summary>
        /// <param name="env"></param>
        /// <returns></returns>
        private Task ProcessRequest(IDictionary<string, object> env)
        {

            // 从字典中获取向客户（浏览器）发送数据的“流”对象
            /////////////////////////////////////////////////////////
            var responseStream = env["owin.ResponseBody"] as Stream;

            // 你准备发送的数据
            const string outString = "<html><head><title>Jexus Owin Server</title></head><body>Jexus Owin Server!<br /><h2>Jexus Owin Server，放飞您灵感的翅膀...</h2>\r\n</body></html>";
            var outBytes = Encoding.UTF8.GetBytes(outString);

            // 从参数字典中获取Response HTTP头的字典对象
            var responseHeaders = env["owin.ResponseHeaders"] as IDictionary<string, string[]>;

            // 设置必要的http响应头
            ////////////////////////////////////////////////////////////////

            // 设置 Content-Type头
            responseHeaders.Add("Content-Type", new[] { "text/html; charset=utf-8" });


            // 把正文写入流中，发送给浏览器
            responseStream.Write(outBytes, 0, outBytes.Length);

            return Task.FromResult<int>(0);

        }


    }



}
